/*******************************************************************************
Copyright (C) 2015 Annapurna Labs Ltd.

This file may be licensed under the terms of the Annapurna Labs Commercial
License Agreement.

Alternatively, this file can be distributed under the terms of the GNU General
Public License V2 as published by the Free Software Foundation and can be
found at http://www.gnu.org/licenses/gpl-2.0.html

Alternatively, redistribution and use in source and binary forms, with or
without modification, are permitted provided that the following conditions are
met:

    *     Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

    *     Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in
the documentation and/or other materials provided with the
distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/

#ifndef __AL_HAL_IOMMU_REGS_H__
#define __AL_HAL_IOMMU_REGS_H__

#ifdef __cplusplus
extern "C" {
#endif
/*
* Unit Registers
*/

struct al_iommu_context_bank_registers {
	uint32_t rsrvd_35[17];
	uint32_t sctlr;          /* Context Bank System Contro ... */
	uint32_t rsrvd_25[7];
	uint32_t ttbr0_low;      /* Context Bank Translation T ... */
	uint32_t ttbr0_high;     /* Context Bank Translation T ... */
	uint32_t rsrvd_26[2];
	uint32_t ttbcr;          /* Context Bank Translation T ... */
	uint32_t rsrvd_27[9];
	uint32_t fsr;            /* Context Bank Fault Status  ... */
	uint32_t fsrrestore;     /* Context Bank Fault Status  ... */
	uint32_t far_low;        /* Context Bank Fault Address ... */
	uint32_t far_high;       /* Context Bank Fault Address ... */
	uint32_t fsynr0;         /* Context Bank Fault Syndrom ... */
	uint32_t rsrvd_28[869];
	uint32_t pmxevcntr[3];   /* Performance Monitor Contex ... */
	uint32_t rsrvd_29[29];
	uint32_t pmxevtyper[3];  /* Performance Monitor Contex ... */
	uint32_t rsrvd_30[29];
	uint32_t pmcfgr;         /* Performance Monitor Contex ... */
	uint32_t pmcr;           /* Performance Monitor Contex ... */
	uint32_t rsrvd_31[6];
	uint32_t pmceid;         /* Performance Monitor Contex ... */
	uint32_t rsrvd_32[7];
	uint32_t pmcntense;      /* Performance Monitor Contex ... */
	uint32_t pmcntenclear;   /* Performance Monitor Contex ... */
	uint32_t pmcntenset;     /* Performance Monitor Contex ... */
	uint32_t pmintenclear;   /* Performance Monitor Contex ... */
	uint32_t pmovsclr;       /* Performance Monitor Contex ... */
	uint32_t rsrvd_33;
	uint32_t pmovsset;       /* Performance Monitor Contex ... */
	uint32_t rsrvd_34[23];
	uint32_t pmauthstatus;   /* Performance Monitor Contex ... */
};

struct al_iommu_non_secure_registers {
	uint32_t iommu_cr0;              /* Non-Secure Configuration r ... */
	uint32_t rsrvd_0[3];
	uint32_t iommu_acr;              /* Non-Secure Auxiliary Confi ... */
	uint32_t rsrvd_1[3];
	uint32_t iommu_idr0;             /* Non-Secure ID register 0 */
	uint32_t iommu_idr1;             /* Non-Secure ID register 1 */
	uint32_t iommu_idr2;             /* Non-Secure ID register 2 */
	uint32_t rsrvd_2[4];
	uint32_t iommu_idr7;             /* Non-Secure ID register 7 */
	uint32_t iommu_gfar_low;         /* Non-Secure Global Fault Ad ... */
	uint32_t iommu_gfar_high;        /* Non-Secure Global Fault Ad ... */
	uint32_t iommu_gfsr;             /* Non-Secure Global Fault St ... */
	uint32_t iommu_gfsrrestore;      /* Non-Secure Global Fault St ... */
	uint32_t iommu_gfsyndr0;         /* Non-Secure Global Fault Sy ... */
	uint32_t iommu_gfsyndr1;         /* Non-Secure Global Fault Sy ... */
	uint32_t rsrvd_3[3];
	uint32_t iommu_tlbivmid;         /* Non-Secure Global Invalida ... */
	uint32_t iommu_tlbiallnsnh;      /* Non-Secure Global TLB inva ... */
	uint32_t rsrvd_4;
	uint32_t iommu_tlbgsync;         /* Non-Secure TLB Global Sync ... */
	uint32_t iommu_tlbgstatus;       /* Non-Secure TLB Global Stat ... */
	uint32_t rsrvd_5[482];
	uint32_t iommu_smr[32];          /* Stream Match register */
	uint32_t rsrvd_6[224];
	uint32_t iommu_s2cr[32];         /* Stream 2 Context register  ... */
	uint32_t rsrvd_7[224];
	uint32_t iommu_cbar[8];          /* Context Bank Attribute r ... */
	uint32_t rsrvd_8[248];
	uint32_t iommu_cbfrsynra[8];     /* Context Bank Fault Restr ... */
	uint32_t rsrvd_9[760];
	uint32_t iommu_intg_ctrl;        /* Integration Control regist ... */
	uint32_t iommu_itip;             /* Integration Input register ... */
	uint32_t iommu_itop;             /* Integration Output registe ... */
	uint32_t rsrvd_10[1021];
	uint32_t iommu_pmxevcntr[3];     /* Performance Monitor Event  ... */
	uint32_t rsrvd_11[253];
	uint32_t iommu_pmxevtyper[3];    /* Performance Monitor Event  ... */
	uint32_t rsrvd_12[253];
	uint32_t iommu_pmcgcr;           /* Performance Monitor Counte ... */
	uint32_t rsrvd_13[127];
	uint32_t iommu_pmcgsmr;          /* Performance Monitor Counte ... */
	uint32_t rsrvd_14[127];
	uint32_t iommu_pmcntenset;       /* Performance Monitor Counte ... */
	uint32_t rsrvd_15[7];
	uint32_t iommu_pmcntenclear;     /* Performance Monitor Counte ... */
	uint32_t rsrvd_16[7];
	uint32_t iommu_pmintenset;       /* Performance Monitor Interr ... */
	uint32_t rsrvd_17[7];
	uint32_t iommu_pmintenclear;     /* Performance Monitor Interr ... */
	uint32_t rsrvd_18[7];
	uint32_t iommu_pmovsclr;         /* Performance Monitor Overfl ... */
	uint32_t rsrvd_19[15];
	uint32_t iommu_pmovsset;         /* Performance Monitor Overfl ... */
	uint32_t rsrvd_20[79];
	uint32_t iommu_pmcfgr;           /* Performance Monitor Config ... */
	uint32_t iommu_pmcr;             /* Performance Monitor Config ... */
	uint32_t rsrvd_21[6];
	uint32_t iommu_pmceid0;          /* Performance Monitor Common ... */
	uint32_t rsrvd_22[101];
	uint32_t iommu_pmauthstatus;     /* Performance Monitor Authen ... */
	uint32_t rsrvd_23[4];
	uint32_t iommu_pmdevtype;        /* Performance Monitor Device ... */
	uint32_t rsrvd_24[4091];
	struct al_iommu_context_bank_registers cb[8];
	uint32_t rsrvd_112[5];
	uint32_t iommu_periph_id4;       /* Peripheral ID register 4 */
	uint32_t iommu_periph_id5;       /* Peripheral ID register 5 */
	uint32_t iommu_periph_id6;       /* Peripheral ID register 6 */
	uint32_t iommu_periph_id7;       /* Peripheral ID register 7 */
	uint32_t iommu_periph_id0;       /* Peripheral ID register 0 */
	uint32_t iommu_periph_id1;       /* Peripheral ID register 1 */
	uint32_t iommu_periph_id2;       /* Peripheral ID register 2 */
	uint32_t iommu_periph_id3;       /* Peripheral ID register 3 */
	uint32_t iommu_comp_id0;         /* Component ID register 0 */
	uint32_t iommu_comp_id1;         /* Component ID register 1 */
	uint32_t iommu_comp_id2;         /* Component ID register 2 */
	uint32_t iommu_comp_id3;         /* Component ID register 3 */
};

struct al_iommu_regs {
	struct al_iommu_non_secure_registers non_secure_registers;
};


/*
* Registers Fields
*/


/**** iommu_cr0 register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CR0_CLIENTPD (1 << 0)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CR0_GFRE (1 << 1)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CR0_GFIE (1 << 2)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CR0_GCFGFRE (1 << 4)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CR0_GCFGFIE (1 << 5)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CR0_STALLD (1 << 8)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CR0_GSE (1 << 9)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CR0_USFCFG (1 << 10)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CR0_VMIDPNE (1 << 11)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CR0_PTM (1 << 12)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CR0_FB (1 << 13)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CR0_BSU_MASK 0x0000C000
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CR0_BSU_SHIFT 14

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CR0_MEMATTR_MASK 0x000F0000
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CR0_MEMATTR_SHIFT 16

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CR0_MTCFG (1 << 20)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CR0_SMCFCFG (1 << 21)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CR0_SHCFG_MASK 0x00C00000
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CR0_SHCFG_SHIFT 22

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CR0_RACFG_MASK 0x03000000
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CR0_RACFG_SHIFT 24

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CR0_WACFG_MASK 0x0C000000
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CR0_WACFG_SHIFT 26

/**** iommu_acr register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_ACR_PREFETCH (1 << 0)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_ACR_WC1EN (1 << 1)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_ACR_WC2EN (1 << 2)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_ACR_SMTNMC_BPTLBEN (1 << 8)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_ACR_MMUDIS_BPTLBEN (1 << 9)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_ACR_S2CR_BPTLBEN (1 << 10)

/**** iommu_idr0 register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_IDR0_NUMSMRG_MASK 0x000000FF
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_IDR0_NUMSMRG_SHIFT 0

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_IDR0_NUMSIDB_MASK 0x00001E00
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_IDR0_NUMSIDB_SHIFT 9

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_IDR0_BTM (1 << 13)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_IDR0_CTTW (1 << 14)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_IDR0_NUMIRPT_MASK 0x00FF0000
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_IDR0_NUMIRPT_SHIFT 16

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_IDR0_PTFS (1 << 24)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_IDR0_SMS (1 << 27)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_IDR0_NTS (1 << 28)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_IDR0_S2TS (1 << 29)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_IDR0_S1TS (1 << 30)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_IDR0_SES (1 << 31)

/**** iommu_idr1 register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_IDR1_NUMCB_MASK 0x000000FF
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_IDR1_NUMCB_SHIFT 0

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_IDR1_NUMSSDNDXB_MASK 0x00000F00
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_IDR1_NUMSSDNDXB_SHIFT 8

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_IDR1_SSDTP (1 << 12)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_IDR1_SMCD (1 << 13)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_IDR1_NUMS2CB_MASK 0x00FF0000
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_IDR1_NUMS2CB_SHIFT 16

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_IDR1_NUMPAGENDXB_MASK 0x70000000
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_IDR1_NUMPAGENDXB_SHIFT 28

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_IDR1_PAGESIZE (1 << 31)

/**** iommu_idr2 register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_IDR2_IAS_MASK 0x0000000F
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_IDR2_IAS_SHIFT 0

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_IDR2_OAS_MASK 0x000000F0
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_IDR2_OAS_SHIFT 4

/**** iommu_idr7 register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_IDR7_MINOR_MASK 0x0000000F
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_IDR7_MINOR_SHIFT 0

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_IDR7_MAJOR_MASK 0x000000F0
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_IDR7_MAJOR_SHIFT 4

/**** iommu_gfar_high register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_GFAR_HIGH_BITS_MASK 0x000000FF
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_GFAR_HIGH_BITS_SHIFT 0

/**** iommu_gfsr register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_GFSR_ICF (1 << 0)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_GFSR_USF (1 << 1)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_GFSR_SMCF (1 << 2)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_GFSR_UCBF (1 << 3)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_GFSR_UCIF (1 << 4)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_GFSR_CAF (1 << 5)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_GFSR_EF (1 << 6)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_GFSR_PF (1 << 7)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_GFSR_MULTI (1 << 31)

/**** iommu_gfsyndr0 register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_GFSYNDR0_NESTED (1 << 0)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_GFSYNDR0_WNR (1 << 1)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_GFSYNDR0_PNU (1 << 2)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_GFSYNDR0_IND (1 << 3)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_GFSYNDR0_NSSTATE (1 << 4)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_GFSYNDR0_NSATTR (1 << 5)

/**** iommu_gfsyndr1 register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_GFSYNDR1_STREAM_ID_MASK 0x00000FFF
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_GFSYNDR1_STREAM_ID_SHIFT 0

/**** iommu_tlbivmid register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_TLBIVMID_VMID_MASK 0x000000FF
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_TLBIVMID_VMID_SHIFT 0

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_TLBIVMID_BITS_MASK 0xFFFFFF00
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_TLBIVMID_BITS_SHIFT 8

/**** iommu_tlbgstatus register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_TLBGSTATUS_GSACTIVE (1 << 0)

/**** iommu_smr register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_SMR_ID_MASK 0x00000FFF
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_SMR_ID_SHIFT 0

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_SMR_MASK_MASK 0x0FFF0000
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_SMR_MASK_SHIFT 16

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_SMR_VALID (1 << 31)

/**** iommu_s2cr register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_S2CR_CBNDX_MASK 0x00000007
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_S2CR_CBNDX_SHIFT 0

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_S2CR_SHCFG_MASK 0x00000300
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_S2CR_SHCFG_SHIFT 8

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_S2CR_MTCFG (1 << 11)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_S2CR_MEM_ATTR_MASK 0x0000F000
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_S2CR_MEM_ATTR_SHIFT 12

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_S2CR_TYPE_MASK 0x00030000
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_S2CR_TYPE_SHIFT 16
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_S2CR_TYPE_TRANSLATION \
	(0x0 << IOMMU_NON_SECURE_REGISTERS_IOMMU_S2CR_TYPE_SHIFT)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_S2CR_NSCFG_MASK 0x000C0000
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_S2CR_NSCFG_SHIFT 18

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_S2CR_RACFG_MASK 0x00300000
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_S2CR_RACFG_SHIFT 20

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_S2CR_WACFG_MASK 0x00C00000
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_S2CR_WACFG_SHIFT 22

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_S2CR_PRIVCFG_MASK 0x03000000
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_S2CR_PRIVCFG_SHIFT 24

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_S2CR_INSTCFG_MASK 0x0C000000
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_S2CR_INSTCFG_SHIFT 26

/**** iommu_cbar register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CBAR_VMID_MASK 0x000000FF
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CBAR_VMID_SHIFT 0

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CBAR_IRPTNDX_MASK 0xFF000000
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CBAR_IRPTNDX_SHIFT 24

/**** iommu_cbfrsynra register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CBFRSYNRA_STREAMID_MASK 0x00000FFF
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CBFRSYNRA_STREAMID_SHIFT 0

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CBFRSYNRA_SSD_INDEX_MASK 0x7FFF0000
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CBFRSYNRA_SSD_INDEX_SHIFT 16

/**** iommu_intg_ctrl register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_INTG_CTRL_INTG_TEST_EN (1 << 0)

/**** iommu_itip register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_ITIP_SPINDEN (1 << 0)

/**** iommu_itop register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_ITOP_NSCFG_IRPT (1 << 0)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_ITOP_NSGLB_IRPT (1 << 1)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_ITOP_PERF_IRPT (1 << 2)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_ITOP_CXT_IRPT (1 << 3)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_ITOP_SCFG_IRPT (1 << 8)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_ITOP_SGLB_IRPT (1 << 9)

/**** iommu_pmxevtyper register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMXEVTYPER_EVENT_MASK 0x0000001F
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMXEVTYPER_EVENT_SHIFT 0

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMXEVTYPER_NSUNPRV (1 << 28)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMXEVTYPER_NSPRV (1 << 29)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMXEVTYPER_UNPRV (1 << 30)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMXEVTYPER_PRV (1 << 31)

/**** iommu_pmcgcr register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCGCR_NDX_MASK 0x00000007
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCGCR_NDX_SHIFT 0

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCGCR_TCEFCFG_MASK 0x00000300
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCGCR_TCEFCFG_SHIFT 8

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCGCR_CBAEN (1 << 10)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCGCR_CEN (1 << 11)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCGCR_EX (1 << 12)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCGCR_SIDG_MASK 0x007F0000
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCGCR_SIDG_SHIFT 16

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCGCR_CGNC_MASK 0x0F000000
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCGCR_CGNC_SHIFT 24

/**** iommu_pmcgsmr register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCGSMR_ID_MASK 0x00000FFF
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCGSMR_ID_SHIFT 0

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCGSMR_MASK_MASK 0x0FFF0000
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCGSMR_MASK_SHIFT 16

/**** iommu_pmcntenset register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCNTENSET_ENB_MASK 0x00000007
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCNTENSET_ENB_SHIFT 0

/**** iommu_pmcntenclear register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCNTENCLEAR_DISAB_MASK 0x00000007
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCNTENCLEAR_DISAB_SHIFT 0

/**** iommu_pmintenset register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMINTENSET_ENB_MASK 0x00000007
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMINTENSET_ENB_SHIFT 0

/**** iommu_pmintenclear register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMINTENCLEAR_DISAB_MASK 0x00000007
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMINTENCLEAR_DISAB_SHIFT 0

/**** iommu_pmovsclr register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMOVSCLR_ENB_MASK 0x00000007
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMOVSCLR_ENB_SHIFT 0

/**** iommu_pmovsset register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMOVSSET_ENB_MASK 0x00000007
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMOVSSET_ENB_SHIFT 0

/**** iommu_pmcfgr register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCFGR_NUM_CNT_MASK 0x000000FF
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCFGR_NUM_CNT_SHIFT 0

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCFGR_RAO_MASK 0x00000700
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCFGR_RAO_SHIFT 8

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCFGR_SIZE_MASK 0x00001800
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCFGR_SIZE_SHIFT 11

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCFGR_RAZ (1 << 13)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCFGR_CC (1 << 14)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCFGR_CCD (1 << 15)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCFGR_EX (1 << 16)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCFGR_NA (1 << 17)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCFGR_WT (1 << 18)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCFGR_UEN (1 << 19)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCFGR_NCG_MASK 0xFF000000
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCFGR_NCG_SHIFT 24

/**** iommu_pmcr register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCR_CEN (1 << 0)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCR_RST (1 << 1)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCR_CCR (1 << 2)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCR_CCD (1 << 3)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCR_EX (1 << 4)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCR_DP (1 << 5)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCR_NA (1 << 6)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCR_WT (1 << 7)

/**** iommu_pmceid0 register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCEID0_CC (1 << 0)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCEID0_REFILL (1 << 1)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCEID0_RD_REFILL (1 << 2)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCEID0_WR_REFILL (1 << 3)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCEID0_ACCESS (1 << 4)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCEID0_RD_ACCESS (1 << 5)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMCEID0_WR_ACCESS (1 << 6)

/**** iommu_pmauthstatus register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMAUTHSTATUS_NSE (1 << 0)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMAUTHSTATUS_NSI (1 << 1)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMAUTHSTATUS_NSNE (1 << 2)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMAUTHSTATUS_NSNI (1 << 3)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMAUTHSTATUS_SE (1 << 4)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMAUTHSTATUS_SI (1 << 5)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMAUTHSTATUS_SNE (1 << 6)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMAUTHSTATUS_SNI (1 << 7)

/**** iommu_pmdevtype register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMDEVTYPE_C_MASK 0x0000000F
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMDEVTYPE_C_SHIFT 0

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMDEVTYPE_T_MASK 0x000000F0
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PMDEVTYPE_T_SHIFT 4

/**** iommu_cb_sctlr register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_SCTLR_MMU (1 << 0)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_SCTLR_TRE (1 << 1)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_SCTLR_AFE (1 << 2)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_SCTLR_AFFD (1 << 3)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_SCTLR_ENDIAN (1 << 4)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_SCTLR_CFRE (1 << 5)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_SCTLR_CFIE (1 << 6)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_SCTLR_CFCFG (1 << 7)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_SCTLR_HUPCF (1 << 8)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_SCTLR_PTW (1 << 13)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_SCTLR_BSU_MASK 0x0000C000
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_SCTLR_BSU_SHIFT 14

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_SCTLR_MEM_ATTR_MASK 0x000F0000
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_SCTLR_MEM_ATTR_SHIFT 16

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_SCTLR_FB (1 << 21)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_SCTLR_SHCFG_MASK 0x00C00000
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_SCTLR_SHCFG_SHIFT 22

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_SCTLR_RACFG_MASK 0x03000000
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_SCTLR_RACFG_SHIFT 24

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_SCTLR_WACFG_MASK 0x0C000000
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_SCTLR_WACFG_SHIFT 26

/**** iommu_cb_ttbr0_high register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_TTBR0_HIGH_BITS_MASK 0x000000FF
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_TTBR0_HIGH_BITS_SHIFT 0

/**** iommu_cb_ttbcr register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_TTBCR_T0SZ_MASK 0x0000000F
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_TTBCR_T0SZ_SHIFT 0
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_TTBCR_T0SZ_MINUS_3 \
	(0xD << IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_TTBCR_T0SZ_SHIFT)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_TTBCR_S (1 << 4)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_TTBCR_SL0_MASK 0x000000C0
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_TTBCR_SL0_SHIFT 6
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_TTBCR_SL0_LEVEL2 \
	(0x0 << IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_TTBCR_SL0_SHIFT)
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_TTBCR_SL0_LEVEL1 \
	(0x1 << IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_TTBCR_SL0_SHIFT)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_TTBCR_IRGN0_MASK 0x00000300
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_TTBCR_IRGN0_SHIFT 8
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_TTBCR_IRGN0_NON_CACHEABLE \
	(0x0 << IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_TTBCR_IRGN0_SHIFT)
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_TTBCR_IRGN0_WRITEBACK_WRITE_ALLOCATE \
	(0x1 << IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_TTBCR_IRGN0_SHIFT)
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_TTBCR_IRGN0_WRITE_THROUGH \
	(0x2 << IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_TTBCR_IRGN0_SHIFT)
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_TTBCR_IRGN0_WRITEBACK_NO_WRITE_ALLOCATE \
	(0x3 << IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_TTBCR_IRGN0_SHIFT)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_TTBCR_ORGN0_MASK 0x00000C00
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_TTBCR_ORGN0_SHIFT 10
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_TTBCR_ORGN0_NON_CACHEABLE \
	(0x0 << IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_TTBCR_ORGN0_SHIFT)
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_TTBCR_ORGN0_WRITEBACK_WRITE_ALLOCATE \
	(0x1 << IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_TTBCR_ORGN0_SHIFT)
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_TTBCR_ORGN0_WRITE_THROUGH \
	(0x2 << IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_TTBCR_ORGN0_SHIFT)
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_TTBCR_ORGN0_WRITEBACK_NO_WRITE_ALLOCATE \
	(0x3 << IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_TTBCR_ORGN0_SHIFT)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_TTBCR_SH0_MASK 0x00003000
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_TTBCR_SH0_SHIFT 12
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_TTBCR_SH0_NON_SHAREABLE \
	(0x0 << IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_TTBCR_SH0_SHIFT)
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_TTBCR_SH0_OUTER_SHAREABLE \
	(0x2 << IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_TTBCR_SH0_SHIFT)
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_TTBCR_SH0_INNER_SHAREABLE \
	(0x3 << IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_TTBCR_SH0_SHIFT)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_TTBCR_EAE (1 << 31)

/**** iommu_cb_fsr register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_FSR_TF (1 << 1)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_FSR_AFF (1 << 2)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_FSR_PF (1 << 3)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_FSR_EF (1 << 4)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_FSR_TLBMCF (1 << 5)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_FSR_TLBLKF (1 << 6)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_FSR_SS (1 << 30)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_FSR_MULTI (1 << 31)

/**** iommu_cb_far_high register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_FAR_HIGH_BITS_MASK 0x000000FF
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_FAR_HIGH_BITS_SHIFT 0

/**** iommu_cb_fsynr0 register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_FSYNR0_PLVL_MASK 0x00000003
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_FSYNR0_PLVL_SHIFT 0

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_FSYNR0_STAGE (1 << 3)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_FSYNR0_WNR (1 << 4)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_FSYNR0_PNU (1 << 5)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_FSYNR0_IND (1 << 6)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_FSYNR0_NSSTATE (1 << 7)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_FSYNR0_NSATTR (1 << 8)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_FSYNR0_ATOF (1 << 9)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_FSYNR0_PTWF (1 << 10)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_FSYNR0_AFR (1 << 11)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_FSYNR0_S1CBNDX_MASK 0x00FF0000
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_FSYNR0_S1CBNDX_SHIFT 16

/**** iommu_cb_pmxevtype register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMXEVTYPER_EVENT_MASK 0x0000001F
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMXEVTYPER_EVENT_SHIFT 0

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMXEVTYPER_NSUNPRV (1 << 28)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMXEVTYPER_NSPRV (1 << 29)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMXEVTYPER_UNPRV (1 << 30)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMXEVTYPER_PRV (1 << 31)

/**** iommu_cb_pmcfgr register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMCFGR_NUM_CNT_MASK 0x000000FF
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMCFGR_NUM_CNT_SHIFT 0

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMCFGR_RAO_MASK 0x00000700
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMCFGR_RAO_SHIFT 8

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMCFGR_SIZE_MASK 0x00001800
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMCFGR_SIZE_SHIFT 11

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMCFGR_RAZ (1 << 13)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMCFGR_CC (1 << 14)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMCFGR_CCD (1 << 15)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMCFGR_EX (1 << 16)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMCFGR_NA (1 << 17)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMCFGR_WT (1 << 18)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMCFGR_UEN (1 << 19)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMCFGR_NCG_MASK 0xFF000000
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMCFGR_NCG_SHIFT 24

/**** iommu_cb_pmcr register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMCR_CEN (1 << 0)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMCR_RST (1 << 1)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMCR_CCR (1 << 2)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMCR_CCD (1 << 3)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMCR_EX (1 << 4)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMCR_DP (1 << 5)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMCR_NA (1 << 6)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMCR_WT (1 << 7)

/**** iommu_cb_pmceid register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMCEID_CC (1 << 0)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMCEID_REFILL (1 << 1)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMCEID_RD_REFILL (1 << 2)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMCEID_WR_REFILL (1 << 3)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMCEID_ACCESS (1 << 4)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMCEID_RD_ACCESS (1 << 5)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMCEID_WR_ACCESS (1 << 6)

/**** iommu_cb_pmcntense register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMCNTENSE_ENB_MASK 0x00000007
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMCNTENSE_ENB_SHIFT 0

/**** iommu_cb_pmcntenclear register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMCNTENCLEAR_DISAB_MASK 0x00000007
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMCNTENCLEAR_DISAB_SHIFT 0

/**** iommu_cb_pmcntenset register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMCNTENSET_ENB_MASK 0x00000007
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMCNTENSET_ENB_SHIFT 0

/**** iommu_cb_pmintenclear register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMINTENCLEAR_DISAB_MASK 0x00000007
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMINTENCLEAR_DISAB_SHIFT 0

/**** iommu_cb_pmovsclr register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMOVSCLR_ENB_MASK 0x00000007
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMOVSCLR_ENB_SHIFT 0

/**** iommu_cb_pmovsset register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMOVSSET_ENB_MASK 0x00000007
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMOVSSET_ENB_SHIFT 0

/**** iommu_cb_pmauthstatus register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMAUTHSTATUS_NSE (1 << 0)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMAUTHSTATUS_NSI (1 << 1)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMAUTHSTATUS_NSNE (1 << 2)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMAUTHSTATUS_NSNI (1 << 3)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMAUTHSTATUS_SE (1 << 4)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMAUTHSTATUS_SI (1 << 5)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMAUTHSTATUS_SNE (1 << 6)

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_CB_PMAUTHSTATUS_SNI (1 << 7)

/**** iommu_periph_id register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PERIPH_ID_PART0_MASK 0x0000000F
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PERIPH_ID_PART0_SHIFT 0

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PERIPH_ID_PART1_MASK 0x000000F0
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_PERIPH_ID_PART1_SHIFT 4

/**** iommu_comp_id register ****/

#define IOMMU_NON_SECURE_REGISTERS_IOMMU_COMP_ID_PREAMBLE_MASK 0x000000FF
#define IOMMU_NON_SECURE_REGISTERS_IOMMU_COMP_ID_PREAMBLE_SHIFT 0

#ifdef __cplusplus
}
#endif

#endif /* __AL_hal_iommu_REGS_H */




